/*

	Idea:
	
	A company has:
	Items, that are collection of Subitems
	Subitems, that have a value that can change over time	
	For each Item the company receive a payment, one or more time during a year, up until the overall value of 
	all subitems of an item is reached

*/

use [tempdb]
go

alter database TemporalSnapshotFact set single_user with rollback immediate
go

drop database TemporalSnapshotFact
go

create database TemporalSnapshotFact
go

use TemporalSnapshotFact
go

create schema util
go

create schema etl
go

create schema tmp
go

create schema dwh
go

create schema staging
go

create schema olap
go

create function [util].[fn_Nums](@m as bigint) returns table
as
return
with
t0 as (select n = 1 union all select n = 1),
t1 as (select n = 1 from t0 as a, t0 as b),
t2 as (select n = 1 from t1 as a, t1 as b),
t3 as (select n = 1 from t2 as a, t2 as b),
t4 as (select n = 1 from t3 as a, t3 as b),
t5 as (select n = 1 from t4 as a, t4 as b),
rslt as (select row_number() over (order by n) as n from t5)
select n from rslt where n <= @m
go

create table staging.items
(
	[item_id] int not null,
	[status] char(1) not null,	
	[valid_from] date not null,
	[valid_to] date not null,	
	constraint pk__items primary key ([item_id], [valid_from])
)
go

create table staging.subitems
(
	[item_id] int not null,
	[subitem_id] int not null,
	[status] char(1) not null,	
	[valid_from] date not null,
	[valid_to] date not null,	
	[value] money not null,
	constraint pk__subitems primary key ([item_id], [subitem_id], [valid_from])
)
go

create table staging.items_payments
(
	[item_id] int not null,	
	[payment_date] date not null,	
	[value] money not null,
	constraint pk__items_payments primary key ([item_id], [payment_date])
)
go

insert into staging.items values
(1, 'A', '20081101', '20090101'),
(1, 'B', '20090101', '20100301'),
(1, 'C', '20100301', '99991231'),
(2, 'A', '20081201', '20090101'),
(2, 'B', '20090101', '20090501'),
(2, 'C', '20090501', '20091001'),
(2, 'D', '20091001', '20091201'),
(2, 'C', '20091201', '99991231')
go

insert into staging.subitems values
(1, 1, 'A', '20081101', '20090101', 1000),
(1, 1, 'B', '20090101', '20090401', 2000),
(1, 1, 'C', '20090401', '20090701', 2000),
(1, 1, 'D', '20090701', '20100101', 2000),
(1, 1, 'B', '20100101', '20100301', 3000),
(1, 1, 'C', '20100301', '99991231', 3000),

(1, 2, 'A', '20081101', '20090101', 1000),
(1, 2, 'B', '20090101', '20090701', 2000),
(1, 2, 'C', '20090701', '20091001', 2000),
(1, 2, 'D', '20091001', '20091201', 2500),
(1, 2, 'E', '20091201', '20100201', 3000),
(1, 2, 'C', '20100201', '99991231', 4000),

(2, 1, 'A', '20081201', '20090101', 1000),
(2, 1, 'B', '20090101', '20090501', 1000),
(2, 1, 'C', '20090501', '20091001', 1000),
(2, 1, 'D', '20091001', '20091201', 1000),
(2, 1, 'C', '20091201', '99991231', 1000)
go

insert into staging.items_payments values
(1, '20081101', 1500), 
(1, '20081210', 1500),
(1, '20090203', 1000),
(1, '20090920', 1200),
(1, '20100116', 1300),
(1, '20100219', 1000),
(1, '20100220', 500),
(2, '20080601', 200),
(2, '20081201', 200),
(2, '20090601', 200),
(2, '20091201', 200),
(2, '20100601', 200),
(2, '20101201', 200)
go