use [TemporalSnapshotFact]
go

select * from staging.items where item_id = 1
select * from staging.subitems where item_id = 1
select * from staging.items_payments where item_id = 1
go

-- how many items are in status "B" and have a subitem with status "C" on August 10th, 2009?
select * from staging.items where [status] = 'B' and '20090810' >= valid_from and '20090810' < valid_to;
select * from staging.subitems where [status] = 'C' and '20090810' >= valid_from and '20090810' < valid_to;

-- what's the status of payments up to that date?
select * from staging.items_payments where payment_date <= '20090810';

--let's put all togheter!

--let's start with a first, simple, approach
with cte1 as
(
	select * from staging.items where [status] = 'B' and '20090810' >= valid_from and '20090810' < valid_to
),
cte2 as
(
	select * from staging.subitems where [status] = 'C' and '20090810' >= valid_from and '20090810' < valid_to
)
select
	*
from
	cte1
inner join
	cte2 on [cte1].[item_id] = [cte2].[item_id];
	
--this approach works well, but each time the analysis date change, query must be re-executed...
--so this is not a good approach to create a data warehouse.

--let's change the approach, so that we can first create a table with all the data
--that can be consolidated and used for every query

--let's start joining only the two temporal tables
--for simplicity focus on one specific subitem
--first problem: calculate the new range 
select
	joined_valid_from = case when s.valid_from > i.valid_from then s.valid_from else i.valid_from end,
	joined_valid_to = case when s.valid_to < i.valid_to then s.valid_to else i.valid_to end,
	*
from	
	staging.items i
inner join
	staging.subitems s on i.item_id = s.item_id 
where
	i.valid_from < s.valid_to and s.valid_from < i.valid_to -- OVERLAPS
and
	i.item_id = 1 and s.subitem_id = 2

-- now let's make thing a little bit more complex: let's add also the payment table
-- problem: notice the row 1 and 2 or 6 and 7: new ranges has to be created in order to correctly
-- represent the reality! How would you that?
;with cte as
(
select
	joined_valid_from = case when s.valid_from > i.valid_from then s.valid_from else i.valid_from end,
	joined_valid_to = case when s.valid_to < i.valid_to then s.valid_to else i.valid_to end,
	i.item_id,
	s.subitem_id,
	item_status = i.[status],
	subitem_status = s.status,
	s.[value]
from	
	staging.items i
inner join
	staging.subitems s on i.item_id = s.item_id 
where
	i.valid_from < s.valid_to and s.valid_from < i.valid_to -- OVERLAPS
and
	i.item_id = 1 and s.subitem_id = 2	
)
select	
	c.*,
	p.payment_date,
	p.[value]
from
	cte c
inner join
	staging.items_payments p on c.item_id = p.item_id
where
	p.payment_date >= c.[joined_valid_from] and p.payment_date < c.[joined_valid_to]
	

-- In reality we would have more than two temporal table to join!
-- How can we deal with such complex situation?